==> localstack_extension_event_studio-0.1.dev1.tar.gz: localstack_extension_event_studio-0.1.dev1/backend/eventstudio/db/event_storage.py <==
_Q='span_id'
_P='sqlalchemy.orm'
_O='sqlalchemy.dialects'
_N='sqlalchemy.pool'
_M='sqlalchemy.engine.Engine'
_L='sqlalchemy.engine'
_K='sqlalchemy'
_J='standard'
_I='error'
_H=True
_G='propagate'
_F='WARNING'
_E='level'
_D='console'
_C=False
_B='handlers'
_A=None
import logging.config
from typing import Dict,List
import sqlalchemy as sa
from sqlalchemy import desc
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session
from eventstudio.api.event_streamer import EventStreamer
from eventstudio.db.models import ErrorDBModel,EventDBModel,TraceLinkDBModel,get_engine
from eventstudio.tracing.context import TraceContext
from eventstudio.types.errors import InputErrorModel
from eventstudio.types.events import EventModel,EventModelList,InputEventModel
from eventstudio.types.responses import Error,FailedEntry
from eventstudio.types.services import ServiceName
logging.config.dictConfig({'version':1,'disable_existing_loggers':_C,'formatters':{_J:{'format':'%(asctime)s %(levelname)s %(name)s: %(message)s'}},_B:{_D:{'class':'logging.StreamHandler','formatter':_J,_E:_F}},'loggers':{_K:{_B:[_D],_E:_F,_G:_C},_L:{_B:[_D],_E:_F,_G:_C},_M:{_B:[_D],_E:_F,_G:_C},_N:{_B:[_D],_E:_F,_G:_C},_O:{_B:[_D],_E:_F,_G:_C},_P:{_B:[_D],_E:_F,_G:_C}}})
logging.getLogger(_K).setLevel(logging.WARNING)
logging.getLogger(_L).setLevel(logging.WARNING)
logging.getLogger(_M).setLevel(logging.WARNING)
logging.getLogger(_N).setLevel(logging.WARNING)
logging.getLogger(_O).setLevel(logging.WARNING)
logging.getLogger(_P).setLevel(logging.WARNING)
LOG=logging.getLogger(__name__)
class EventStorageService:
	def __init__(A,db_path:str='',event_streamer:EventStreamer|_A=_A):A._engine=get_engine(db_path);(A._event_streaming_service):EventStreamer|_A=event_streamer
	def close_connection(A):
		if A._engine:A._engine.dispose()
	def add_event(C,event:InputEventModel)->tuple[str,str]|Error:
		A=event
		try:
			with Session(C._engine)as D:
				B=EventDBModel(**A.model_dump());D.add(B);D.commit()
				if C._event_streaming_service:A=EventModel.model_validate(B,from_attributes=_H);A.event_bytedata=_A;C._event_streaming_service.notify(A)
				return B.span_id,B.trace_id
		except Exception as E:LOG.error(f"Failed to add event: {E}");return{_I:E}
	def add_error(A,error:InputErrorModel):
		with Session(A._engine)as B:
			C=ErrorDBModel(**error.model_dump());B.add(C);B.commit()
			if A._event_streaming_service:D=C.span_id;E=A.get_event(D);A._event_streaming_service.notify(E)
	def list_events(A)->List[EventModel]:
		C=A._fetch_all_events();A._remove_binary_data(C);D=A._map_child_events_to_parent_events(C);B=A._add_direct_children(C,D)
		for E in B:A._get_latest_event(E)
		B=A._remove_hidden_events(B);B=A._combine_lambda_invoke_response_events(B);return B
	def list_events_v2(A)->List[EventModel]:
		B=A.list_events();D={A.span_id:A for A in B};C=[]
		for E in B:A._create_remapped_rows(E,D,C)
		return C
	def get_event(D,span_id:str)->EventModel|_A:
		A=span_id
		try:
			with Session(D._engine)as E:
				B=E.query(EventDBModel).filter(EventDBModel.span_id==A).first()
				if not B:return
				return EventModel.model_validate(B)
		except Exception as C:LOG.error(f"Failed to get event: {C}");return{_I:C,_Q:A}
	def delete_event(D,span_id:str)->FailedEntry|_A:
		A=span_id
		try:
			with Session(D._engine)as B:E=sa.delete(EventDBModel).where(EventDBModel.span_id==A);B.execute(E);B.commit()
		except Exception as C:LOG.error(f"Failed to delete event: {C}");return{_I:C,_Q:A}
	def delete_all_events(C)->Error|_A:
		try:
			with Session(C._engine)as A:A.query(EventDBModel).delete();A.commit()
		except Exception as B:LOG.error(f"Failed to delete all events: {B}");return{_I:B}
	def list_events_graph(D)->EventModelList:
		with Session(D._engine)as E:A=E.query(EventDBModel).all();A=[EventModel.model_validate(A)for A in A]
		C={A.span_id:A for A in A}
		for B in A:B.event_bytedata=_A
		for B in A:
			if B.parent_id and B.parent_id in C:F=C[B.parent_id];F.children.append(B)
		return EventModelList(events=A)
	def get_event_graph(C,trace_id:str|_A)->EventModel|_A:
		with Session(C._engine)as G:B=G.query(EventDBModel).where(EventDBModel.trace_id==trace_id).all();B=[EventModel.model_validate(A)for A in B]
		C._remove_binary_data(B);F=_A;D={}
		for A in B:
			if A.parent_id is _A:F=A
			D[A.span_id]=A
		for A in B:
			if A.parent_id is _A:continue
			if A.parent_id in D:D[A.parent_id].children.append(A)
		E=C._get_latest_event(F);E=C._process_lambda_events(E);return E
	def store_xray_trace(F,xray_trace_id:str,trace_context:TraceContext)->_A|Error:
		D=trace_context;B=xray_trace_id
		try:
			with Session(F._engine)as E:G=TraceLinkDBModel(xray_trace_id=B,parent_id=D.parent_id,trace_id=D.trace_id);E.add(G);E.commit();return
		except SQLAlchemyError as C:A=f"Database error while storing XRay trace {B}: {str(C)}";LOG.error(A);return Error(error=A)
		except Exception as C:A=f"Unexpected error while storing XRay trace {B}: {str(C)}";LOG.error(A);return Error(error=A)
	def get_trace_from_xray_trace(E,xray_trace_id:str)->tuple[str,str]|_A|Error:
		B=xray_trace_id
		try:
			with Session(E._engine)as F:
				C=F.query(TraceLinkDBModel).filter(TraceLinkDBModel.xray_trace_id==B).order_by(desc(TraceLinkDBModel.recording_time)).first()
				if not C:LOG.debug(f"No trace link found for XRay trace ID: {B}");return
				return C.trace_id,C.parent_id
		except SQLAlchemyError as D:A=f"Database error while retrieving event for XRay trace {B}: {str(D)}";LOG.error(A);return Error(error=A)
		except Exception as D:A=f"Unexpected error while retrieving event for XRay trace {B}: {str(D)}";LOG.error(A);return Error(error=A)
	def get_event_by_id(D,event_id:str)->EventModel|Error|_A:
		A=event_id
		try:
			with Session(D._engine)as E:
				B=E.query(EventDBModel).filter(EventDBModel.event_id==A).order_by(EventDBModel.creation_time.desc()).first()
				if not B:return
				return EventModel.model_validate(B)
		except Exception as F:C=f"Failed to get event with message_id {A}: {F}";LOG.error(C);return Error(error=C)
	def _fetch_all_events(A)->List[EventModel]:
		with Session(A._engine)as B:C=B.query(EventDBModel).all();return[EventModel.model_validate(A)for A in C]
	def _remove_binary_data(B,events:List[EventModel])->_A:
		for A in events:A.event_bytedata=_A
	def _map_child_events_to_parent_events(C,events:List[EventModel])->Dict[str,List[EventModel]]:
		B={}
		for A in events:
			if A.parent_id:
				if A.parent_id not in B:B[A.parent_id]=[]
				B[A.parent_id].append(A)
		return B
	def _add_direct_children(E,all_events:List[EventModel],children_by_parent:Dict[str,List[EventModel]])->List[EventModel]:
		B=[]
		for C in all_events:
			A=C.model_copy();A.children=children_by_parent.get(C.span_id,[])
			for D in A.children:D.children=[]
			B.append(A)
		return B
	def _create_remapped_rows(E,event:EventModel,events_by_span_id:Dict[str,EventModel],rows:List[EventModel])->_A:
		D=events_by_span_id;A=event
		if A.parent_id is _A:rows.append(A.model_copy(deep=_H))
		elif A.parent_id in D:B=D[A.parent_id].model_copy(deep=_H);C=A.model_copy(deep=_H);B.operation_name=C.operation_name;B.children=[C];B.errors=C.errors;rows.append(B)
	def _get_latest_event(C,event:EventModel)->EventModel:
		A=event
		if not A.children:return A
		B=max(A.children,key=lambda e:e.version)
		if B.version>A.version and B.operation_name=='replay_event':
			if not B.children:return B
			A=B.children[0].model_copy(deep=_H);A.version=B.version
		A.children=[C._get_latest_event(A)for A in A.children];return A
	def _remove_hidden_events(I,events:List[EventModel])->List[EventModel]:
		B=events;D=[A for A in B if A.is_hidden]
		if D:
			E=[A for A in B if not A.is_hidden];F={A.span_id:A for A in E}
			for C in D:
				A=C.parent_id
				if A and A in F:
					for G in C.children:G.parent_id=A
					H=F[A];H.children=C.children
			return E
		return B
	def _is_lambda_invoke_event(B,event:EventModel)->bool:A=event;return A.service==ServiceName.LAMBDA and A.operation_name=='Invoke'and A.event_data and A.event_data.payload is not _A and A.event_data.response is _A
	def _is_lambda_response_event(B,event:EventModel)->bool:A=event;return A.service==ServiceName.LAMBDA and A.operation_name=='Response'and A.event_data and A.event_data.response is not _A
	def _combine_lambda_invoke_response_events(C,events:List[EventModel])->List[EventModel]:
		D=events
		for A in D:
			if C._is_lambda_invoke_event(A):
				E=A.event_metadata.function_name
				if E:
					for(F,B)in enumerate(A.children):
						if C._is_lambda_response_event(B):
							if B.event_metadata.function_name==E:B=A.children.pop(F);A.event_data.response=B.event_data.response;break
		return D
	def _process_lambda_events(C,event:EventModel)->EventModel:
		B=event;F=C._is_lambda_invoke_event(B);E=set()
		if F:
			for A in B.children:
				if C._is_lambda_response_event(A):B.event_data.response=A.event_data.response;E.add(A.span_id)
		D=[]
		for A in B.children:
			if A.span_id in E:D.extend([C._process_lambda_events(A)for A in A.children]);continue
			G=C._process_lambda_events(A);D.append(G)
		B.children=D;return B

==> localstack_extension_event_studio-0.1.3.tar.gz: localstack_extension_event_studio-0.1.3/backend/eventstudio/db/event_storage.py <==
_Q='span_id'
_P='sqlalchemy.orm'
_O='sqlalchemy.dialects'
_N='sqlalchemy.pool'
_M='sqlalchemy.engine.Engine'
_L='sqlalchemy.engine'
_K='sqlalchemy'
_J='standard'
_I='error'
_H=True
_G='propagate'
_F='WARNING'
_E='level'
_D='console'
_C=False
_B='handlers'
_A=None
import logging.config
from typing import Dict,List
import sqlalchemy as sa
from sqlalchemy import desc
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session
from eventstudio.api.event_streamer import EventStreamer
from eventstudio.db.models import ErrorDBModel,EventDBModel,TraceLinkDBModel,get_engine
from eventstudio.tracing.context import TraceContext
from eventstudio.types.errors import InputErrorModel
from eventstudio.types.events import EventModel,EventModelList,InputEventModel
from eventstudio.types.responses import Error,FailedEntry
from eventstudio.types.services import ServiceName
logging.config.dictConfig({'version':1,'disable_existing_loggers':_C,'formatters':{_J:{'format':'%(asctime)s %(levelname)s %(name)s: %(message)s'}},_B:{_D:{'class':'logging.StreamHandler','formatter':_J,_E:_F}},'loggers':{_K:{_B:[_D],_E:_F,_G:_C},_L:{_B:[_D],_E:_F,_G:_C},_M:{_B:[_D],_E:_F,_G:_C},_N:{_B:[_D],_E:_F,_G:_C},_O:{_B:[_D],_E:_F,_G:_C},_P:{_B:[_D],_E:_F,_G:_C}}})
logging.getLogger(_K).setLevel(logging.WARNING)
logging.getLogger(_L).setLevel(logging.WARNING)
logging.getLogger(_M).setLevel(logging.WARNING)
logging.getLogger(_N).setLevel(logging.WARNING)
logging.getLogger(_O).setLevel(logging.WARNING)
logging.getLogger(_P).setLevel(logging.WARNING)
LOG=logging.getLogger(__name__)
class EventStorageService:
	def __init__(A,db_path:str='',event_streamer:EventStreamer|_A=_A):A._engine=get_engine(db_path);(A._event_streaming_service):EventStreamer|_A=event_streamer
	def close_connection(A):
		if A._engine:A._engine.dispose()
	def add_event(C,event:InputEventModel)->tuple[str,str]|Error:
		A=event
		try:
			with Session(C._engine)as D:
				B=EventDBModel(**A.model_dump());D.add(B);D.commit()
				if C._event_streaming_service:A=EventModel.model_validate(B,from_attributes=_H);A.event_bytedata=_A;C._event_streaming_service.notify(A)
				return B.span_id,B.trace_id
		except Exception as E:LOG.error(f"Failed to add event: {E}");return{_I:E}
	def add_error(B,error:InputErrorModel):
		with Session(B._engine)as A:C=ErrorDBModel(**error.model_dump());A.add(C);A.commit()
	def list_db_events(B)->List[EventModel]:
		A=B._fetch_all_events()
		if not A:return[]
		B._remove_binary_data(A);return A
	def list_events(A)->List[EventModel]:
		C=A._fetch_all_events()
		if not C:return[]
		A._remove_binary_data(C);D=A._map_child_events_to_parent_events(C);B=A._add_direct_children(C,D)
		for E in B:A._get_latest_event(E)
		B=A._combine_lambda_invoke_response_events(B);B=A._remove_hidden_events(B);return B
	def list_events_v2(A)->List[EventModel]:
		B=A._fetch_all_events()
		if not B:return[]
		A._remove_binary_data(B);G=A._map_child_events_to_parent_events(B)
		for C in B:C.children=G.get(C.span_id,[])
		for C in B:A._get_latest_event(C)
		D=A._combine_lambda_invoke_response_events(B);D=A._remove_hidden_events(D);E=[];H=set()
		for C in D:
			F=A._process_service_chain(C,H)
			if F:E.append(F)
		I={A.span_id:A for A in D};J=A._create_remapped_rows(E,I);return J
	def get_event(D,span_id:str)->EventModel|_A:
		A=span_id
		try:
			with Session(D._engine)as E:
				B=E.query(EventDBModel).filter(EventDBModel.span_id==A).first()
				if not B:return
				return EventModel.model_validate(B)
		except Exception as C:LOG.error(f"Failed to get event: {C}");return{_I:C,_Q:A}
	def delete_event(D,span_id:str)->FailedEntry|_A:
		A=span_id
		try:
			with Session(D._engine)as B:E=sa.delete(EventDBModel).where(EventDBModel.span_id==A);B.execute(E);B.commit()
		except Exception as C:LOG.error(f"Failed to delete event: {C}");return{_I:C,_Q:A}
	def delete_all_events(C)->Error|_A:
		try:
			with Session(C._engine)as A:A.query(EventDBModel).delete();A.commit()
		except Exception as B:LOG.error(f"Failed to delete all events: {B}");return{_I:B}
	def list_events_graph(D)->EventModelList:
		with Session(D._engine)as E:A=E.query(EventDBModel).all();A=[EventModel.model_validate(A)for A in A]
		C={A.span_id:A for A in A}
		for B in A:B.event_bytedata=_A
		for B in A:
			if B.parent_id and B.parent_id in C:F=C[B.parent_id];F.children.append(B)
		return EventModelList(events=A)
	def get_event_graph(C,trace_id:str|_A)->EventModel|_A:
		with Session(C._engine)as G:B=G.query(EventDBModel).where(EventDBModel.trace_id==trace_id).all();B=[EventModel.model_validate(A)for A in B]
		C._remove_binary_data(B);F=_A;D={}
		for A in B:
			if A.parent_id is _A:F=A
			D[A.span_id]=A
		for A in B:
			if A.parent_id is _A:continue
			if A.parent_id in D:D[A.parent_id].children.append(A)
		E=C._get_latest_event(F);E=C._process_lambda_events(E);return E
	def store_xray_trace(F,xray_trace_id:str,trace_context:TraceContext)->_A|Error:
		D=trace_context;B=xray_trace_id
		try:
			with Session(F._engine)as E:G=TraceLinkDBModel(trace_link=B,parent_id=D.parent_id,trace_id=D.trace_id);E.add(G);E.commit();return
		except SQLAlchemyError as C:A=f"Database error while storing XRay trace {B}: {str(C)}";LOG.error(A);return Error(error=A)
		except Exception as C:A=f"Unexpected error while storing XRay trace {B}: {str(C)}";LOG.error(A);return Error(error=A)
	def get_trace_from_xray_trace(E,xray_trace_id:str)->tuple[str,str]|_A|Error:
		B=xray_trace_id
		try:
			with Session(E._engine)as F:
				C=F.query(TraceLinkDBModel).filter(TraceLinkDBModel.trace_link==B).order_by(desc(TraceLinkDBModel.recording_time)).first()
				if not C:LOG.debug(f"No trace link found for XRay trace ID: {B}");return
				return C.trace_id,C.parent_id
		except SQLAlchemyError as D:A=f"Database error while retrieving event for XRay trace {B}: {str(D)}";LOG.error(A);return Error(error=A)
		except Exception as D:A=f"Unexpected error while retrieving event for XRay trace {B}: {str(D)}";LOG.error(A);return Error(error=A)
	def store_request_id(F,request_id:str,trace_context:TraceContext)->_A|Error:
		D=trace_context;B=request_id
		try:
			with Session(F._engine)as E:G=TraceLinkDBModel(trace_link=B,parent_id=D.parent_id,trace_id=D.trace_id);E.add(G);E.commit();return
		except SQLAlchemyError as C:A=f"Database error while storing Request ID trace {B}: {str(C)}";LOG.error(A);return Error(error=A)
		except Exception as C:A=f"Unexpected error while storing Request ID trace {B}: {str(C)}";LOG.error(A);return Error(error=A)
	def get_trace_from_request_id(E,request_id:str)->tuple[str,str]|_A|Error:
		B=request_id
		try:
			with Session(E._engine)as F:
				C=F.query(TraceLinkDBModel).filter(TraceLinkDBModel.trace_link==B).order_by(desc(TraceLinkDBModel.recording_time)).first()
				if not C:LOG.debug(f"No trace link found for Request ID: {B}");return
				return C.trace_id,C.parent_id
		except SQLAlchemyError as D:A=f"Database error while retrieving event for Request ID {B}: {str(D)}";LOG.error(A);return Error(error=A)
		except Exception as D:A=f"Unexpected error while retrieving event for Request ID {B}: {str(D)}";LOG.error(A);return Error(error=A)
	def get_event_by_id(D,event_id:str)->EventModel|Error|_A:
		A=event_id
		try:
			with Session(D._engine)as E:
				B=E.query(EventDBModel).filter(EventDBModel.event_id==A).order_by(EventDBModel.creation_time.desc()).first()
				if not B:return
				return EventModel.model_validate(B)
		except Exception as F:C=f"Failed to get event with message_id {A}: {F}";LOG.error(C);return Error(error=C)
	def _fetch_all_events(A)->List[EventModel]:
		with Session(A._engine)as B:C=B.query(EventDBModel).all();return[EventModel.model_validate(A)for A in C]
	def _remove_binary_data(B,events:List[EventModel])->_A:
		for A in events:A.event_bytedata=_A
	def _map_child_events_to_parent_events(C,events:List[EventModel])->Dict[str,List[EventModel]]:
		B={}
		for A in events:
			if A.parent_id:
				if A.parent_id not in B:B[A.parent_id]=[]
				B[A.parent_id].append(A)
		return B
	def _add_direct_children(E,all_events:List[EventModel],children_by_parent:Dict[str,List[EventModel]])->List[EventModel]:
		B=[]
		for C in all_events:
			A=C.model_copy();A.children=children_by_parent.get(C.span_id,[])
			for D in A.children:D.children=[]
			B.append(A)
		return B
	def _get_latest_event(C,event:EventModel)->EventModel:
		A=event
		if not A.children:return A
		B=max((A for A in A.children if A.operation_name=='replay_event'),key=lambda e:e.version,default=_A)
		if B and B.version>A.version:
			if not B.children:return B
			A=B.children[0].model_copy(deep=_H);A.version=B.version
		A.children=[C._get_latest_event(A)for A in A.children];return A
	def _remove_hidden_events(I,events:List[EventModel])->List[EventModel]:
		D=events;F=[A for A in D if A.is_hidden]
		if F:
			G=[A for A in D if not A.is_hidden];H={A.span_id:A for A in G}
			for A in F:
				B=A.parent_id
				if B and B in H:
					for C in A.children:C.parent_id=B
					E=H[B]
					for C in E.children:
						if C.span_id==A.span_id:E.children.remove(C);break
					E.children.extend(A.children)
			return G
		return D
	def _is_lambda_invoke_event(B,event:EventModel)->bool:A=event;return A.service==ServiceName.LAMBDA and A.operation_name=='Invoke'and A.event_data and A.event_data.payload is not _A and A.event_data.response is _A
	def _is_lambda_response_event(B,event:EventModel)->bool:A=event;return A.service==ServiceName.LAMBDA and A.operation_name=='Response'and A.event_data and A.event_data.response is not _A
	def _combine_lambda_invoke_response_events(C,events:List[EventModel])->List[EventModel]:
		D=events
		for A in D:
			if C._is_lambda_invoke_event(A):
				E=A.event_metadata.function_name
				if E:
					for(F,B)in enumerate(A.children):
						if C._is_lambda_response_event(B):
							if B.event_metadata.function_name==E:B=A.children.pop(F);A.event_data.response=B.event_data.response;break
		return D
	def _process_lambda_events(C,event:EventModel)->EventModel:
		B=event;F=C._is_lambda_invoke_event(B);E=set()
		if F:
			for A in B.children:
				if C._is_lambda_response_event(A):B.event_data.response=A.event_data.response;E.add(A.span_id)
		D=[]
		for A in B.children:
			if A.span_id in E:D.extend([C._process_lambda_events(A)for A in A.children]);continue
			G=C._process_lambda_events(A);D.append(G)
		B.children=D;return B
	def _process_service_chain(G,event:EventModel,visited)->EventModel|_A:
		B=visited;A=event
		if A.span_id in B:return
		H=A.arn;D=[];E=[]
		def F(current_event):
			A=current_event
			if A.span_id in B:return
			if A.arn==H:
				D.append(A);B.add(A.span_id)
				for C in A.children:F(C)
			else:E.append(A)
		F(A);C=G._combine_events(D)
		if C:C.children=E
		return C
	def _combine_events(E,events:List[EventModel])->EventModel|_A:
		A=events
		if not A:return
		C=[]
		for D in A:
			if D.errors:C.extend(D.errors)
		B=A[-1].model_copy(deep=_H);B.errors=C;B.children=[];return B
	def _create_remapped_rows(G,events:List[EventModel],events_by_span_id:Dict[str,EventModel])->List[EventModel|_A]:
		D=[]
		for B in events:
			if not B.children:continue
			if B and B.children:
				for(E,F)in enumerate(B.children):A=B.model_copy(deep=_H);C=F.model_copy(deep=_H);A.operation_name=C.operation_name;A.event_id=f"{A.event_id}_{E}";C.children=[];A.children=[C];A.errors.extend(C.errors);D.append(A)
		return D
